clear all

% Exemple of NL-InSAR
%
%       AUTHOR:
%               Charles Deledalle
%               email: deledalle@telecom-paristech.fr
%
%       VERSION: 23 August 2010

%%% Load and generate noise-free images of InSAR parameters
house = double(imread('house.png'));
R = house.^2;
R(R == 0) = 1;
B = mod(2*pi*(house/256)-pi, 2*pi)-pi;
D = (house+64)/(255+64);

%%% Generate a noisy pair of SLC SAR images
[z1, z2] = insarrnd(R, B, D);

%%% Parameters
hW = 10;                % search window of size (2hW+1)^2
hD = 3;                 % patches of size (2hD+1)^2
Lmin = 10;              % Minimum equivalent number of looks
nbit = 10;              % Number of iterations

%%% Single Look Maximum Likelihood (ML)
Rmv = (abs(z1).^2 + abs(z2).^2) / 2;
Dmv = abs(z1 .* conj(z2)) ./ Rmv;
Bmv = angle(z1 .* conj(z2));

%%% NL-InSAR
tic;
    R_nlinsar = ones(size(z1));
    B_nlinsar = zeros(size(z1));
    D_nlinsar = zeros(size(z1));
    for i = 1:nbit
        fprintf(1, 'Iteration %d/%d\n', i, nbit);
        [R_nlinsar, B_nlinsar, D_nlinsar] = ...
            nlinsar(z1, z2, ...
                    R_nlinsar, B_nlinsar, D_nlinsar, ...
                    hW, hD, ...
                    Lmin);
    end
toc

%%% Show results
figure,
ax(1) = subplot(3, 1, 1);
plotimage_insar(R, B, D, R);
title('Noise-free images (reflectivity, phase and coherence)');
ax(2) = subplot(3, 1, 2);
plotimage_insar(Rmv, Bmv, Dmv, R);
title(sprintf('Noisy images (SNR: %.2f, %.2f and %.2f)', ...
              snr(Rmv, R), ...
              isnr(Bmv, B), ...
              snr(Dmv, D)));
ax(3) = subplot(3, 1, 3);
plotimage_insar(R_nlinsar, B_nlinsar, D_nlinsar, R);
title(sprintf('Nl-InSAR estimation (SNR: %.2f, %.2f and %.2f)', ...
              snr(R_nlinsar, R), ...
              isnr(B_nlinsar, B), ...
              snr(D_nlinsar, D)));
linkaxes(ax);
